@@ -37,14 +37,19 @@ G_BEGIN_DECLS
 
 /**
  * PeasEngine:
- * @parent: the parent object.
  *
  * Engine at the heart of the Peas plugin system.
  */
-typedef struct _PeasEngine        PeasEngine;
-typedef struct _PeasEngineClass   PeasEngineClass;
-typedef struct _PeasEnginePrivate PeasEnginePrivate;
+typedef struct _PeasEngine         PeasEngine;
+typedef struct _PeasEngineClass    PeasEngineClass;
+typedef struct _PeasEnginePrivate  PeasEnginePrivate;
 
+/**
+ * PeasEngine:
+ *
+ * The #PeasEngine structure contains only private data and should only be
+ * accessed using the provided API.
+ */
 struct _PeasEngine {
   GObject parent;
 
@@ -88,9 +93,24 @@ void              peas_engine_garbage_collect     (PeasEngine      *engine);
 gboolean          peas_engine_provides_extension  (PeasEngine      *engine,
                                                    PeasPluginInfo  *info,
                                                    GType            extension_type);
-PeasExtension    *peas_engine_get_extension       (PeasEngine      *engine,
+
+
+PeasExtension    *peas_engine_create_extensionv   (PeasEngine      *engine,
                                                    PeasPluginInfo  *info,
-                                                   GType            extension_type);
+                                                   GType            extension_type,
+                                                   guint            n_parameters,
+                                                   GParameter      *parameters);
+PeasExtension    *peas_engine_create_extension_valist
+                                                  (PeasEngine      *engine,
+                                                   PeasPluginInfo  *info,
+                                                   GType            extension_type,
+                                                   const gchar     *first_property,
+                                                   va_list          var_args);
+PeasExtension    *peas_engine_create_extension    (PeasEngine      *engine,
+                                                   PeasPluginInfo  *info,
+                                                   GType            extension_type,
+                                                   const gchar     *first_property,
+                                                   ...);
 
 
 G_END_DECLS
