@@ -28,6 +28,7 @@
 #include "peas-extension-set.h"
 #include "peas-plugin-info.h"
 #include "peas-marshal.h"
+#include "peas-helpers.h"
 
 /**
  * SECTION:peas-extension-set
@@ -42,8 +43,57 @@
  * #PeasExtensionSet will automatically track loading and unloading of
  * the plugins, and signal appearance and disappearance of new
  * extension instances.  You should connect to those signals if you
- * wish to call specific methods on loading or unloading time, as it
- * is typically the case for #PeasActivatable extensions.
+ * wish to call specific methods on loading or unloading time.
+ *
+ * Here is the code for a typical setup of #PeasExtensionSet with
+ * #PeasActivatable as the watched extension point, and #GtkWindow
+ * instances as the target objects:
+ * |[
+ * static void
+ * on_extension_added (PeasExtensionSet *set,
+ *                     PeasPluginInfo   *info,
+ *                     PeasExtension    *exten,
+ *                     GtkWindow        *window)
+ * {
+ *   peas_extension_call (exten, "activate", window);
+ * }
+ *
+ * static void
+ * on_extension_removed (PeasExtensionSet *set,
+ *                       PeasPluginInfo   *info,
+ *                       PeasExtension    *exten,
+ *                       GtkWindow        *window)
+ * {
+ *   peas_extension_call (exten, "deactivate", window);
+ * }
+ *
+ * static gboolean
+ * on_delete_event (GtkWidget        *window,
+ *                  GdkEvent         *event,
+ *                  PeasExtensionSet *set)
+ * {
+ *   peas_extension_set_call (set, "deactivate");
+ *   return FALSE;
+ * }
+ *
+ * PeasExtensionSet *
+ * setup_extension_set (PeasEngine *engine,
+ *                      GtkWindow  *window)
+ * {
+ *   PeasExtensionSet *set;
+ *
+ *   set = peas_extension_set_new (engine, PEAS_TYPE_ACTIVATABLE,
+ *                                 "object", window, NULL);
+ *   peas_extension_set_call (set, "activate");
+ *   g_signal_connect (set, "extension-added",
+ *                     G_CALLBACK (on_extension_added), window);
+ *   g_signal_connect (set, "extension-removed",
+ *                     G_CALLBACK (on_extension_removed), window);
+ *   g_signal_connect (window, "delete-event",
+ *                     G_CALLBACK (on_delete_event), set);
+ *   return set;
+ * }
+ * ]|
  **/
 
 G_DEFINE_TYPE (PeasExtensionSet, peas_extension_set, G_TYPE_OBJECT);
@@ -51,6 +101,9 @@ G_DEFINE_TYPE (PeasExtensionSet, peas_extension_set, G_TYPE_OBJECT);
 struct _PeasExtensionSetPrivate {
   PeasEngine *engine;
   GType exten_type;
+  guint n_parameters;
+  GParameter *parameters;
+
   GList *extensions;
 
   gulong load_handler_id;
@@ -75,9 +128,27 @@ static guint signals[LAST_SIGNAL];
 enum {
   PROP_0,
   PROP_ENGINE,
-  PROP_EXTENSION_TYPE
+  PROP_EXTENSION_TYPE,
+  PROP_CONSTRUCT_PROPERTIES
 };
 
+static void
+set_construct_properties (PeasExtensionSet *set,
+                          PeasParameterArray *array)
+{
+  unsigned i;
+
+  set->priv->n_parameters = array->n_parameters;
+
+  set->priv->parameters = g_new0 (GParameter, array->n_parameters);
+  for (i = 0; i < array->n_parameters; i++)
+    {
+      set->priv->parameters[i].name = g_intern_string (array->parameters[i].name);
+      g_value_init (&set->priv->parameters[i].value, G_VALUE_TYPE (&array->parameters[i].value));
+      g_value_copy (&array->parameters[i].value, &set->priv->parameters[i].value);
+    }
+}
+
 static void
 peas_extension_set_set_property (GObject      *object,
                                  guint         prop_id,
@@ -95,6 +166,9 @@ peas_extension_set_set_property (GObject      *object,
     case PROP_EXTENSION_TYPE:
       set->priv->exten_type = g_value_get_gtype (value);
       break;
+    case PROP_CONSTRUCT_PROPERTIES:
+      set_construct_properties (set, g_value_get_pointer (value));
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
     }
@@ -132,13 +206,14 @@ add_extension (PeasExtensionSet *set,
   if (!peas_plugin_info_is_loaded (info))
     return;
 
-  exten = peas_engine_get_extension (set->priv->engine, info,
-                                     set->priv->exten_type);
+  exten = peas_engine_create_extensionv (set->priv->engine, info,
+                                         set->priv->exten_type,
+                                         set->priv->n_parameters,
+                                         set->priv->parameters);
   if (!exten)
     return;
 
 /*  peas_plugin_info_ref (info); */
-  g_object_ref (exten);
 
   item = (ExtensionItem *) g_slice_new (ExtensionItem);
   item->info = info;
@@ -221,6 +296,13 @@ peas_extension_set_finalize (GObject *object)
       l = g_list_delete_link (l, l);
     }
 
+  if (set->priv->parameters != NULL)
+    {
+      while (set->priv->n_parameters-- > 0)
+        g_value_unset (&set->priv->parameters[set->priv->n_parameters].value);
+      g_free (set->priv->parameters);
+    }
+
   g_object_unref (set->priv->engine);
 }
 
@@ -255,6 +337,21 @@ peas_extension_set_class_init (PeasExtensionSetClass *klass)
 
   klass->call = peas_extension_set_call_real;
 
+  /**
+   * PeasExtensionSet::extension-added:
+   * @set: A #PeasExtensionSet.
+   * @info: A #PeasPluginInfo.
+   * @exten: A #PeasExtension.
+   *
+   * The extension-added signal is emitted when a new extension has been
+   * added to the #PeasExtensionSet. It happens when a new plugin implementing
+   * the extension set's extension type is loaded.
+   *
+   * You should connect to this signal in order to set up the extensions when
+   * they are loaded. Note that this signal is not fired for extensions coming
+   * from plugins that were already loaded when the #PeasExtensionSet instance
+   * was created. You should set those up by yourself.
+   */
   signals[EXTENSION_ADDED] =
     g_signal_new ("extension-added",
                   the_type,
@@ -267,6 +364,21 @@ peas_extension_set_class_init (PeasExtensionSetClass *klass)
                   PEAS_TYPE_PLUGIN_INFO | G_SIGNAL_TYPE_STATIC_SCOPE,
                   PEAS_TYPE_EXTENSION);
 
+  /**
+   * PeasExtensionSet::extension-removed:
+   * @set: A #PeasExtensionSet.
+   * @info: A #PeasPluginInfo.
+   * @exten: A #PeasExtension.
+   *
+   * The extension-added signal is emitted when a new extension is about to be
+   * removed from the #PeasExtensionSet. It happens when a plugin implementing
+   * the extension set's extension type is unloaded.
+   *
+   * You should connect to this signal in order to clean up the extensions
+   * when their plugin is unload. Note that this signal is not fired for the
+   * #PeasExtension instances still available when the #PeasExtensionSet
+   * instance is destroyed. You should clean those up by yourself.
+   */
   signals[EXTENSION_REMOVED] =
     g_signal_new ("extension-removed",
                   the_type,
@@ -297,6 +409,14 @@ peas_extension_set_class_init (PeasExtensionSetClass *klass)
                                                        G_PARAM_CONSTRUCT_ONLY |
                                                        G_PARAM_STATIC_STRINGS));
 
+  g_object_class_install_property (object_class, PROP_CONSTRUCT_PROPERTIES,
+                                   g_param_spec_pointer ("construct-properties",
+                                                         "Construct Properties",
+                                                         "The properties to pass the extensions when creating them",
+                                                         G_PARAM_WRITABLE |
+                                                         G_PARAM_CONSTRUCT_ONLY |
+                                                         G_PARAM_STATIC_STRINGS));
+
   g_type_class_add_private (klass, sizeof (PeasExtensionSetPrivate));
 }
 
@@ -329,7 +449,7 @@ peas_extension_set_call (PeasExtensionSet *set,
 
 /**
  * peas_extension_set_call_valist:
- * @exten: A #PeasExtensionSet.
+ * @set: A #PeasExtensionSet.
  * @method_name: the name of the method that should be called.
  * @args: the arguments for the method.
  *
@@ -354,19 +474,111 @@ peas_extension_set_call_valist (PeasExtensionSet *set,
 }
 
 /**
- * peas_extension_set_new:
+ * peas_extension_set_newv:
  * @engine: A #PeasEngine.
  * @exten_type: the extension #GType.
+ * @n_parameters: the length of the @parameters array.
+ * @parameters: an array of #GParameter.
  *
- * Create an #ExtensionSet for all the @exten_type extensions defined in
- * the plugins loaded in @engine.
+ * Create a new #PeasExtensionSet for the @exten_type extension type.
+ *
+ * See peas_extension_set_new() for more information.
+ *
+ * Returns: a new instance of #PeasExtensionSet.
  */
 PeasExtensionSet *
-peas_extension_set_new (PeasEngine *engine,
-                        GType       exten_type)
+peas_extension_set_newv (PeasEngine *engine,
+                         GType       exten_type,
+                         guint       n_parameters,
+                         GParameter *parameters)
 {
+  PeasParameterArray construct_properties = { n_parameters, parameters };
+
   return PEAS_EXTENSION_SET (g_object_new (PEAS_TYPE_EXTENSION_SET,
                                            "engine", engine,
                                            "extension-type", exten_type,
+                                           "construct-properties", &construct_properties,
                                            NULL));
 }
+
+/**
+ * peas_extension_set_new_valist:
+ * @engine: A #PeasEngine.
+ * @exten_type: the extension #GType.
+ * @first_property: the name of the first property.
+ * @var_args: the value of the first property, followed optionally by more
+ *   name/value pairs, followed by %NULL.
+ *
+ * Create a new #PeasExtensionSet for the @exten_type extension type.
+ *
+ * See peas_extension_set_new() for more information.
+ *
+ * Returns: a new instance of #PeasExtensionSet.
+ */
+PeasExtensionSet *
+peas_extension_set_new_valist (PeasEngine  *engine,
+                               GType        exten_type,
+                               const gchar *first_property,
+                               va_list      var_args)
+{
+  gpointer type_struct;
+  GParameter *parameters;
+  guint n_parameters;
+  PeasExtensionSet *set;
+
+  type_struct = _g_type_struct_ref (exten_type);
+
+  if (!_valist_to_parameter_list (exten_type, type_struct, first_property,
+                                  var_args, &parameters, &n_parameters))
+    {
+      /* WARNING */
+      g_return_val_if_reached (NULL);
+    }
+
+  set = peas_extension_set_newv (engine, exten_type, n_parameters, parameters);
+
+  while (n_parameters-- > 0)
+    g_value_unset (&parameters[n_parameters].value);
+  g_free (parameters);
+
+  _g_type_struct_unref (exten_type, type_struct);
+
+  return set;
+}
+
+/**
+ * peas_extension_set_new:
+ * @engine: A #PeasEngine.
+ * @exten_type: the extension #GType.
+ * @first_property: the name of the first property.
+ * @Varargs: the value of the first property, followed optionally by more
+ *   name/value pairs, followed by %NULL.
+ *
+ * Create a new #PeasExtensionSet for the @exten_type extension type.
+ *
+ * At any moment, the #PeasExtensionSet will contain an extension instance for
+ * each loaded plugin which implements the @exten_type extension type. It does
+ * so by connecting to the relevant signals from #PeasEngine.
+ *
+ * The property values passed to peas_extension_set_new() will be used for the
+ * construction of new extension instances.
+ *
+ * See peas_engine_create_extension() for more information.
+ *
+ * Returns: a new instance of #PeasExtensionSet.
+ */
+PeasExtensionSet *
+peas_extension_set_new (PeasEngine  *engine,
+                        GType        exten_type,
+                        const gchar *first_property,
+                        ...)
+{
+  va_list var_args;
+  PeasExtensionSet *set;
+
+  va_start (var_args, first_property);
+  set = peas_extension_set_new_valist (engine, exten_type, first_property, var_args);
+  va_end (var_args);
+
+  return set;
+}
